#include "stdio.h"
#include "benewake_lidar.h"
#include "can.h"
#include "string.h"
#include "stdlib.h"
#include "sys.h"
#include "usart.h"
#include "timer.h"
#include "led.h"

#ifdef LIDAR_ON_FLAG
static volatile int g_lidar_on_flag = 0;
#else
static volatile int g_lidar_on_flag = 1;
#endif

extern AUX_TxBuff_TypeDef g_TxBuff;

// ״У׼غ
void lidar_enable_center_calibration(void);
void lidar_disable_center_calibration(void);
void lidar_center_calibration(uint8_t center, u8 ProtocolMode);

// ״־λغ
void lidar_set_error_flag(uint8_t flag);
void lidar_clear_error_flag(uint8_t flag);

// ״джϺ
void lidar_is_error(uint8_t status);

/* lidar
 * @ line_numberbit7-6: á  
                  00 -8й 01 - 2й10-16й11-24й
 * @ mode: 000-ģʽ  001-У׼
 * @ Xcֵ  λcm
 * @ Zc: ֵ  λcm
 * @ fps: ֡
 */
int lidar_start(u8 line_number, u8 mode, u16 Xc, u16 Zc, u8 fps)
{
    u8 canData[5];
    if(g_lidar_on_flag == 0)
    {
        return ERR_LOST_CONNECT;
    }

    canData[0] = line_number << 6 | (fps << 1) | 0x01;
    canData[1] = 0x00;
    canData[2] = Xc & 0xff;
    canData[3] = Zc / 10;
    if (mode == 0x01)
    {
        lidar_enable_center_calibration();
        canData[4] = 0x5a;
    }
    else
    {
        canData[4] = 0xaa;
    }
    CAN1_Send(canData, sizeof(canData), TO_LIADAR_CMD_ID);
    return ACK;
}

/*
* ڹ̶ʱδյ
*  = 0
*/
void lidar_on_flag_reset(void)
{
    printf("Lost Connection\r\n");
    g_lidar_on_flag = 0;
}

/*
* ״led
*/
void lidar_on_flag_LED(void)
{
    static uint8_t led_count = 0;
    if(g_lidar_on_flag == 0)
    {
        LED_CE30_ERROR = 1;
        LED_CE30_ONLINE = 0;
    }
    else
    {
        LED_CE30_ERROR = 0;
        if(led_count == 100)
        {
            LED_CE30_ONLINE =~ LED_CE30_ONLINE;
            led_count = 0;
        }
        else
        {
            led_count++;
        }
    }
}

/*
* ֹͣlidar
*
*/
int lidar_stop(void)
{
    u8 canData[5];
    if(g_lidar_on_flag == 0)
    {
        return ERR_LOST_CONNECT;
    }
    canData[0] = 0x00;    // bit0: 1 -    0 - ֹͣ
    canData[1] = 0x00;
    canData[2] = 0x00;
    canData[3] = 0x00;
    canData[4] = 0x00;
    CAN1_Send(canData, sizeof(canData), TO_LIADAR_CMD_ID);
    return ACK;
}

// ״ݻbuff
//   lidar_data.data  -->  g_TxBuff.data
// Ƚе״ݷlidar_data.data
// ȴlidar_send_msg
// lidar_data.data˵g_TxBuff.dataв֡ͷ  ɷ
AUX_TxBuff_TypeDef lidar_data;
#define AUX_ADD_HEX_u8(u8data)                  \
{                                               \
    lidar_data.data[lidar_data.idx++] = u8data; \
}

#define AUX_ADD_HEX_u16(u16data)                \
{                                               \
    AUX_ADD_HEX_u8( u16data & 0xff);            \
    AUX_ADD_HEX_u8((u16data>>8) & 0xff);        \
}
/*
* lidarݣϢʱ
* msg: lidarϴdistanceangle
*/
struct nearest_pixel *lidar_recv_msg(CanRxMsg *msg, u8 ProtocolMode)
{
    uint16_t distance1, distance2, distance3;
    int8_t  angle1, angle2, angle3;
    if (msg->StdId == FROM_LIADAR_HEART_ID)
    {
        // ӡCE30Ϣ
        uint8_t ce30_is_run = (msg->Data[0] & 0x01);
        uint8_t version_valid = (msg->Data[0] & 0x02) >> 1;
        uint8_t ce30_status = (msg->Data[0] & 0x04) >> 2;
        uint8_t ce30_center = (msg->Data[0] & 0x10) >> 4;
        uint8_t value = (msg->Data[0] & 0xE0) >> 5;
        if(version_valid == 0x01)
        {
            uint8_t version = msg->Data[1];
            printf("is_run = %x, is_error = %x, is_center = %x, value = %x, version = %x\r\n",ce30_is_run, ce30_status, ce30_center, value, version);
        }
        else
        {
            printf("is_run = %x, is_error = %x, is_center = %x, value = %x\r\n",ce30_is_run, ce30_status, ce30_center, value);
        }
        g_lidar_on_flag = 1;
        #ifdef LIDAR_ON_FLAG
        TIM_Cmd(TIM3, ENABLE);
        TIM_ITConfig(TIM3, TIM_IT_Update, ENABLE);
        Timer_SetOnlineCountReset();
        #endif
        
        lidar_is_error(ce30_status);
        lidar_center_calibration(ce30_center, ProtocolMode);

        // Ϣ  ʱ
        return NULL;
    }
    else if ((msg->StdId == FROM_LIADAR_DATA_ID) && (msg->DLC == 8))
    {
        // ģʽ
        LED_CE30_DATA = 1;
        Timer_SetDataCountReset();
        if(ProtocolMode == protocol_FALSHHOLD)
        {
            distance1 = msg->Data[1] << 8 | msg->Data[0];
            angle1 = msg->Data[3];
            AUX_ADD_HEX_u16(distance1);
            AUX_ADD_HEX_u8(angle1);
        }
        else if(ProtocolMode == protocol_MUXING)
        {
            distance1 = msg->Data[2] << 8 | msg->Data[3];
            angle1 = msg->Data[4];
            AUX_ADD_HEX_u16(distance1);
            AUX_ADD_HEX_u8(angle1);
        }
        else if(ProtocolMode == protocol_BENEWAKE)
        {
            distance1 = ((msg->Data[1] & 0x0f) << 8) | (msg->Data[0] & 0xff);
            distance2 = ((msg->Data[2] & 0xff) << 4) | ((msg->Data[1] & 0xf0) >> 4);
            distance3 = ((msg->Data[4] & 0x0f) << 8) | (msg->Data[3] & 0xff);
            angle1 = msg->Data[5];
            angle2 = msg->Data[6];
            angle3 = msg->Data[7];
            AUX_ADD_HEX_u16(distance1);
            AUX_ADD_HEX_u8(angle1);
            AUX_ADD_HEX_u16(distance2);
            AUX_ADD_HEX_u8(angle2);
            AUX_ADD_HEX_u16(distance3);
            AUX_ADD_HEX_u8(angle3);
        }
        return (struct nearest_pixel *)1;
    }
    else
    {
        return NULL;
    }
}
/*
* ֡ͷ
*
*/
void CreateDataHeader(lidar_hdr *header, u8 ProtocolMode)
{
    if(ProtocolMode == protocol_BENEWAKE)
    {
        // ͨЭ
        header->frame_header[0] = 0xaa;
        header->frame_header[1] = 0x55;
        header->mode[0] = 0x03;
        header->mode[1] = 0x00;
    }
    else
    {
        // Э
        header->frame_header[0] = 0xaa;
        header->frame_header[1] = 0x55;
        header->mode[0] = 0x01;
        header->mode[1] = 0x00;
    }
}


/*
* PC˷
*/
void lidar_send_msg(u8 ProtocolMode)
{
    lidar_hdr *header;
    header = (lidar_hdr*)g_TxBuff.data;
    memcpy((void *)&g_TxBuff.data[sizeof(lidar_hdr)], lidar_data.data, lidar_data.idx);
    CreateDataHeader(header, ProtocolMode);
    USART_SendBuff(g_TxBuff.data, lidar_data.idx + sizeof(lidar_hdr));
    lidar_data.idx = 0;
    g_TxBuff.idx = 0;
}



static int center_count = -1;

/*
* ״У׼
*/
void lidar_enable_center_calibration(void)
{
    center_count = 0;
}

/*
* ֹͣ״У׼
*/
void lidar_disable_center_calibration(void)
{
    center_count = -1;
}

/*
* ״У׼֮ڵȴ CENTER_WAIT_NUM_OF_FRAME ֮֡
* ״нȡУ׼
* ж϶Ӧ״̬λϴӦĻָ
*/
void lidar_center_calibration(uint8_t center, u8 ProtocolMode)
{
    uint8_t center_wait_num_of_frame = 0;

    if(center_count != -1)
    {
        center_count++;
        if (protocol_BENEWAKE == ProtocolMode)
        {
            center_wait_num_of_frame = 1;
        }
        else
        {
            center_wait_num_of_frame = 10;
        }
        
        if(center_count == center_wait_num_of_frame)
        {
            if(center == 0x01)
            {
                USART_SendOneByte(ACK);
            }
            else
            {
                USART_SendOneByte(ERR_CENTER);
            }
            lidar_disable_center_calibration();
        }
    }
}

/*
* ж״Ƿ쳣
*/
void lidar_is_error(uint8_t status)
{
    if(status == 1)
    {
        lidar_set_error_flag(ERR_ABNORMAL_FLAG);
    }
    else
    {
        lidar_clear_error_flag(ERR_ABNORMAL_FLAG);
    }
}

static uint8_t lidar_flag = 0;
/*
* ״־λ
*/
void lidar_set_error_flag(uint8_t flag)
{
    lidar_flag |= flag;
}
/*
* ״־λ
*/
void lidar_clear_error_flag(uint8_t flag)
{
    lidar_flag &= ~flag;
}
/*
* ״־λ
*/
uint8_t lidar_get_error_flag(void)
{
    return lidar_flag;
}
